----------------------------------
-- Global Ignore List Variables --
----------------------------------

addonLoaded = false

local gotILupdate    = false
local gotLoaded      = false
local safeToLoad     = false
local faction        = nil
local maxIgnoreSize  = 50
local maxSyncTries   = 1
local maxHistorySize = 100
local firstClear     = false
local firstPrune     = false
local pruneDays      = 0
local timer          = 0
local loadedTime     = GetTime()
local lastSentIgnore = ""

local BlizzardAddIgnore        = nil
local BlizzardDelIgnore        = nil
local BlizzardAddOrDelIgnore   = nil

----------------------------------
-- Global Ignore List Functions --
----------------------------------

local function ShowMsg (msg)

	print ("|cff33ff99Global Ignore: |cffffffff" .. (msg or "nil"))

end

local function OnOff (value)

	if value == nil or value == false then
		return "|cffffff00" .. L["OFF"] .. "|cffffffff"
	elseif value == true then
		return "|cffffff00" .. L["ON"] .. "|cffffffff"
	else
		return "|cffff0000nil"
	end
end

function dayString (value)

	if value == 1 then
		return L["DAY"]
	else
		return L["DAYS"]
	end
end

local function AddToList(newname, newfaction, newnote, newtype)

	local index = #GlobalIgnoreDB.ignoreList+1
	
	GlobalIgnoreDB.ignoreList[index] = newname
	GlobalIgnoreDB.factionList[index] = newfaction
	GlobalIgnoreDB.dateList[index] = date("%d %b %Y")
	GlobalIgnoreDB.notes[index] = (newnote or "")
	GlobalIgnoreDB.expList[index] = GlobalIgnoreDB.defexpire
	GlobalIgnoreDB.typeList[index] = (newtype or "player")
	GlobalIgnoreDB.syncInfo[index] = {}
	
end

local function RemoveFromList(index)

	if index <= #GlobalIgnoreDB.ignoreList then
		table.remove(GlobalIgnoreDB.ignoreList, index)
		table.remove(GlobalIgnoreDB.factionList, index)
		table.remove(GlobalIgnoreDB.dateList, index)
		table.remove(GlobalIgnoreDB.notes, index)
		table.remove(GlobalIgnoreDB.expList, index)
		table.remove(GlobalIgnoreDB.typeList, index)
		table.remove(GlobalIgnoreDB.syncInfo, index)
	end
end

local function getSyncValue (index)
	-- value, index of syncInfo data
		
	local s = ""
	local p = ""
	local v = 0
	
	for c = 1, #GlobalIgnoreDB.syncInfo[index] do
	
		s = GlobalIgnoreDB.syncInfo[index][c]
		p = string.find(s, "@", 1, true)
			
		if p > 0 then
			v = tonumber(string.sub(s, p + 1))
			s = string.sub(s, 1, p - 1)
			
			if playerName == s then
				return v, c				
			end
		end
	end
	
	return 0, 0
end

local function setSyncValue (index)

	local val,idx = getSyncValue(index)
	
	if idx == 0 then
		idx = #GlobalIgnoreDB.syncInfo[index] + 1	
	end
	
	val = val + 1
	
	GlobalIgnoreDB.syncInfo[index][idx] = playerName .. "@" .. val
end		

local function isServerMatch (server1, server2)

	return Proper(server1) == Proper(server2)
	
end

local function hasIgnored (name)

	local result = 0
	
	name = Proper(addServer(name))
		
	for count = 1, GetNumIgnores() do
	
		if name == Proper(addServer(GetIgnoreName(count))) then
			result = count
			
			break
		end
	end
	
	return result
end

function hasNPCIgnored (name)

	if not name then return 0 end
	
	for count = 1, #GlobalIgnoreDB.ignoreList do
	
		if GlobalIgnoreDB.ignoreList[count] == name and GlobalIgnoreDB.typeList[count] == "npc" then
			return count
		end
	end
	
	return 0
end

local function hasServerIgnored (name)

	if not name then return 0 end
	
	for count = 1, #GlobalIgnoreDB.ignoreList do
	
		if GlobalIgnoreDB.ignoreList[count] == name and GlobalIgnoreDB.typeList[count] == "server" then
			return count
		end
	end
	
	return 0
end

function hasGlobalIgnored (name)

	if not name then return 0 end

	for count = 1, #GlobalIgnoreDB.ignoreList do
	
		if GlobalIgnoreDB.ignoreList[count] == name and GlobalIgnoreDB.typeList[count] == "player" then
			return count
		end
	end
	
	return 0
end

function hasAnyIgnored (name)

	if not name then return 0 end

	for count = 1, #GlobalIgnoreDB.ignoreList do
	
		if GlobalIgnoreDB.ignoreList[count] == name then
			return count
		end
	end
	
	return 0
end

local function hasDeleted (name)

	if not name then return 0 end
	
	for count = 1, #GlobalIgnoreDB.delList do
		if GlobalIgnoreDB.delList[count] == name then
			return count
		end
	end
	
	return 0
end

local function addDeleted (name)

	local idx = hasDeleted(name)
	
	if idx == 0 then
		if #GlobalIgnoreDB.delList >= maxHistorySize then
			table.remove(GlobalIgnoreDB.delList, 1)
		end
		
 		GlobalIgnoreDB.delList[#GlobalIgnoreDB.delList + 1] = name
 	end
end

local function removeDeleted (name)

	local idx = hasDeleted(name)
	
	if idx > 0 then
		table.remove(GlobalIgnoreDB.delList, idx)
	end
end

local function ShowIgnoreList (param)

	local days   = tonumber(param)
	local sName  = ""
	
	if not days then
	
		days  = 0
		sName = param
		
		if not sName then
			sName = ""
		end
	end
	
	if days > 0 then
		ShowMsg("|cffffff00" .. format(L["LIST_1"], days))		
	else
		if sName ~= "" then
			sName = Proper(sName)
			
			if sName == "Npc" then
				ShowMsg("|cffffff00".. L["LIST_2"])
			else			
				if sName == "Server" then
					sName = serverName		
				end
				
				ShowMsg("|cffffff00" .. format(L["LIST_3"], sName))
			end
		else
			ShowMsg("|cffffff00" .. L["LIST_4"])
		end
	end
	
	local count = 0
	
	for key,value in pairs(GlobalIgnoreDB.ignoreList) do
	
		local ok   = true
		local type = "P"
		
		if GlobalIgnoreDB.typeList[key] == "npc" then
			type = "N"
		elseif GlobalIgnoreDB.typeList[key] == "server" then
			type = "S"
		end
	
		if days > 0 then
		
			ok = daysFromToday(GlobalIgnoreDB.dateList[key]) >= days
			
		elseif sName ~= "" then
		
			ok = (type == "N" and sName == "Npc") or (type == "P" and isServerMatch(sName, getServer(value))) or (type == "S" and isServerMatch(sName, value))

		end
		
		if ok then
			local str = "  (" .. key .. ") [" .. type.. "] " .. value .. " (" .. (GlobalIgnoreDB.factionList[key] or "Unknown") .. ") " .. "[".. daysFromToday(GlobalIgnoreDB.dateList[key]) .. " "..L["DAYS"] .. "]"
			
			if GlobalIgnoreDB.notes[key] ~= "" then
				str = str .." (" .. GlobalIgnoreDB.notes[key] .. ")"
			end
			
			ShowMsg(str)

			count = count + 1			
		end
		
	end
	
	ShowMsg("|cffffff00" .. format(L["LIST_5"], count))
end

function ResetSpamFilters()

	GlobalIgnoreDB.filterList  = {}
	GlobalIgnoreDB.filterDesc  = {}
	GlobalIgnoreDB.filterCount = {}
	
	GlobalIgnoreDB.invertSpam = false
	GlobalIgnoreDB.spamFilter = true
	
	GlobalIgnoreDB.filterDesc[1]  = "Filter \"Anal\" Spammers"
	GlobalIgnoreDB.filterList[1]  = "([word=anal] or [contains=analan]) and [link]"
	GlobalIgnoreDB.filterCount[1] = 0

	GlobalIgnoreDB.filterDesc[2]  = "Filter Thunderfury linking"
	GlobalIgnoreDB.filterList[2]  = "[item=19019]"
	GlobalIgnoreDB.filterCount[2] = 0
	
end

local function ResetIgnoreDB()

	GlobalIgnoreDB = {
		chatmsg		= true,
		sameserver	= true,
		samefaction	= true,
		openWithFriends	= true,
		attachFriends   = true,
		trackChanges	= true,
		spamFilter	= true,
		invertSpam	= true,
		autoIgnore	= true,
		autoCount	= 3,
		autoTime	= 600,		
		defexpire	= 0,
		ignoreList	= {},
		factionList	= {},
		dateList	= {},
		notes		= {},
		expList		= {},
		typeList	= {},  
		delList		= {},
		syncInfo	= {},
		filterTotal	= 0,
		filterCount	= {},
		filterDesc	= {},
		filterList	= {}		
	}
	
	GlobalIgnoreImported = false
	
	ResetSpamFilters()
end

local function SyncIgnoreList(silent)

	if silent == nil then
		silent = false
	end
	
	-- first remove expired entries
	
	local count = 0
	
	while count < #GlobalIgnoreDB.dateList do
		count = count + 1
		
		if GlobalIgnoreDB.expList[count] > 0 and daysFromToday(GlobalIgnoreDB.dateList[count]) >= GlobalIgnoreDB.expList[count] then
			local name = addServer(GlobalIgnoreDB.ignoreList[count])		
			DelIgnore(name)
			count = 0
		end
	end
	
	-- find account ignores that aren't on global ignore and do things
	
	for count = 1, GetNumIgnores() do
	
		local name       = addServer(GetIgnoreName(count))
		local skipRemove = false
		local globIdx	 = hasGlobalIgnored(name)
	
		if globIdx == 0 then
		
			if GlobalIgnoreDB.trackChanges == true then
			
				local idx = hasDeleted(name)
				
				if idx == 0 then
					skipRemove = true
					AddIgnore(name, true)
				end
			end
			
			if skipRemove == false then

				if not silent then
					ShowMsg (format(L["SYNC_1"], name))
				end			
			
				BlizzardDelIgnore (hasIgnored(name))
			end
		else
			GlobalIgnoreDB.syncInfo[globIdx] = {}
		end
	end
	
	if GlobalIgnoreDB.trackChanges == true then
	
		local listCount = 0
		
		while listCount <= #GlobalIgnoreDB.ignoreList do
		
			if GlobalIgnoreDB.typeList[listCount] == "player" then
			
				local tries = getSyncValue(listCount)

				if tries >= maxSyncTries then
					DelIgnore(removeServer(GlobalIgnoreDB.ignoreList[listCount]))
					listCount = listCount - 1				
				end
			end
			
			listCount = listCount + 1
		end	
	end
	
	-- move qualified players to account wide ignore if there is room for it
		
	local ignoreCount = GetNumIgnores()
	
	if ignoreCount < maxIgnoreSize then

		for key,value in pairs(GlobalIgnoreDB.ignoreList) do
		
			if GlobalIgnoreDB.typeList[key] == "player" then
		
				local name = addServer(value)
				
				if hasIgnored(name) == 0 then
					-- should we add an option to not always samefaction sync?
					local ok = (GlobalIgnoreDB.factionList[key] == faction)

					if ok then
						ok = (isServerMatch(serverName, getServer(name))) or (GlobalIgnoreDB.sameserver == false)
					end
			
					if ok then
						ignoreCount = ignoreCount + 1
						
						setSyncValue(key)
						
						name = removeServer(name)

						if not silent then
							ShowMsg (format(L["SYNC_2"], name))
						end					
								
						BlizzardAddIgnore(name)
					end
				end
			end
			
			if ignoreCount >= maxIgnoreSize then
				break
			end
		end
	end	
end

local function PruneIgnoreList (days, doit)

	if days == nil or days <= 0 then		
		return 0
	end

	local targets = 0
	local count   = 0
	
	while count < #GlobalIgnoreDB.dateList do
		count = count + 1
	
		if daysFromToday(GlobalIgnoreDB.dateList[count]) >= days then
			targets = targets + 1
			
			local name = addServer(GlobalIgnoreDB.ignoreList[count])
					
			if doit == true then
				DelIgnore(name)
				count = 0
			end
		end
	end
	
	return targets
end

local function ApplicationStartup(self)

	if addonLoaded or safeToLoad == false then
		return
	end
	
	ShowMsg(L["LOAD_1"])
		
	faction = UnitFactionGroup("player")
		
	if GlobalIgnoreDB == nil then
		
		ResetIgnoreDB()	
	end
	
	-- set missing defaults or upgrade if needed

	if GlobalIgnoreDB.filterTotal == nil then
		GlobalIgnoreDB.filterTotal = 0
	end

	if GlobalIgnoreDB.spamFilter == nil then
		GlobalIgnoreDB.spamFilter = true
	end
	
	if GlobalIgnoreDB.invertSpam == nil then
		GlobalIgnoreDB.invertSpam = false
	end
	
	if GlobalIgnoreDB.autoIgnore == nil then
		GlobalIgnoreDB.autoIgnore = true
	end
	
	if GlobalIgnoreDB.autoCount == nil then
		GlobalIgnoreDB.autoCount = 3
	end
	
	if GlobalIgnoreDB.autoTime == nil then
		GlobalIgnoreDB.autoTime = 600
	end
	
	if GlobalIgnoreDB.filterList == nil or GlobalIgnoreDB.filterDesc == nil or GlobalIgnoreDB.filterCount == nil then
		ResetSpamFilters()
	end
	
	if GlobalIgnoreDB.delList == nil then
		GlobalIgnoreDB.delList = {}
	end
		
	if GlobalIgnoreDB.defexpire == nil then
		GlobalIgnoreDB.defexpire = 0
	end
	
	if not tonumber(GlobalIgnoreDB.defexpire) then
		GlobalIgnoreDB.defexpire = 0
	end
	
	if GlobalIgnoreDB.trackChanges == nil then
		GlobalIgnoreDB.trackChanges = true
	end
	
	if GlobalIgnoreDB.openWithFriends == nil then
		GlobalIgnoreDB.openWithFriends = true
	end
	
	if not GlobalIgnoreDB.attachFriends == nil then
		GlobalIgnoreDB.attachFriends = true
	end
	
	if GlobalIgnoreDB.asknote == nil then
		GlobalIgnoreDB.asknote = true
	end
		
	if GlobalIgnoreDB.expList == nil then
		GlobalIgnoreDB.expList = {}
			
		for  count = 1, #GlobalIgnoreDB.ignoreList do
			GlobalIgnoreDB.expList[count] = 0
		end
	end
	
	if GlobalIgnoreDB.syncList then
		GlobalIgnoreDB.syncList = nil
	end
	
	if GlobalIgnoreDB.syncInfo == nil then
		GlobalIgnoreDB.syncInfo = {}
		
		for count = 1, #GlobalIgnoreDB.ignoreList do
			GlobalIgnoreDB.syncInfo[count] = {}
		end
	end
	
	if GlobalIgnoreDB.typeList == nil then
		GlobalIgnoreDB.typeList = {}
		
		for count = 1, #GlobalIgnoreDB.ignoreList do
			GlobalIgnoreDB.typeList[count] = "player"
		end
	end
	
	if GlobalIgnoreDB.revision == nil then
		GlobalIgnoreDB.revision = 1
		
		for count = 1, #GlobalIgnoreDB.ignoreList do
			GlobalIgnoreDB.ignoreList[count] = Proper(GlobalIgnoreDB.ignoreList[count])
		end
	end
	
	loadedTime = GetTime()
		
	--
		
	if GlobalIgnoreImported == nil or GlobalIgnoreImported ~= true then
	
		local ignores = GetNumIgnores()	
		local added   = 0
			
		if (ignores > 0) and (GlobalIgnoreDB.chatmsg == nil or GlobalIgnoreDB.chatmsg == true) then
			ShowMsg(L["LOAD_2"])
		end
			
		for count = 1, ignores do
			
			local name = GetIgnoreName(count)
				
			if removeServer(name, true) ~= "Unknown" then
				
				name = Proper(addServer(name))

				if hasGlobalIgnored(name) == 0 then
					added = added + 1
						
					AddToList(name, faction)
										
					if GlobalIgnoreDB.chatmsg == nil or GlobalIgnoreDB.chatmsg == true then
						ShowMsg (format(L["LOAD_3"], name))
					end
				end
			end
				
		end
			
		GlobalIgnoreImported = true
	end
						
	-- Check for GlobalIgnoreDB nil values and set defaults here if needed
		
	SyncIgnoreList(GlobalIgnoreDB.chatmsg == nil or GlobalIgnoreDB.chatmsg == false)
		
	addonLoaded = true
	
	self:UnregisterEvent("IGNORELIST_UPDATE")
	self:UnregisterEvent("ADDON_LOADED")
end

local function EventHandler (self, event, sender, ...)

	--print ("event=".. (event or "nil") .. " sender=" .. (sender or "nil"))
	
	if event == "ADDON_LOADED" and (sender == "GlobalIgnoreList") then
		gotLoaded = true
	end

	if event == "IGNORELIST_UPDATE" then
		gotILupdate = true
	end
	
	if gotLoaded == true and gotILupdate == true then
		safeToLoad = true
	end
	
	if event == "PLAYER_LOGIN" then
		if not safeToLoad and not addonLoaded then
		
			timer = 0
		
			self:SetScript("OnUpdate", function(Self, elapsed)
			
				timer = timer + elapsed
				
				if (gotLoaded ~= true) or (timer < 3) then
					return
				end
												
				self:SetScript("OnUpdate", nil)
				self:Hide()
				
				safeToLoad = true
				
				ApplicationStartup(self)
			end)
		end
	end
	
	if safeToLoad == true then
		ApplicationStartup(self)
	end
end

------------------------
-- SPAM FILTER ENGINE --
------------------------

function filterComplex (filterStr, chatStr)
	-- true=should be filtered

	filterStr = "( " .. filterStr .. " )"

	local chatData	= {}
	local itemID	= {}
	local spellID	= {}
	local achieveID = {}
	local gmatch	= string.gmatch
	local sub	= string.sub
	local find	= string.find
	local lower	= string.lower
	local char	= string.char	
		
	local pos1, pos2
	local tempStr = chatStr
	
	repeat
		pos1, pos2 = find(tempStr, "|Hitem:", 1, true)
		
		if not pos1 then break end

		itemID[#itemID + 1] = sub(tempStr, pos1 + 7, find(tempStr, ":", pos1 + 8, true) - 1, true)
		
		tempStr = sub(tempStr, 1, pos1 - 1) .. sub(tempStr, pos2 + 1)
	until false
	
	tempStr = chatStr

	repeat
		pos1, pos2 = find(tempStr, "|Hspell:", 1, true)
		
		if not pos1 then break end

		spellID[#spellID + 1] = sub(tempStr, pos1 + 8, find(tempStr, "|", pos1 + 9, true) - 1, true)
		
		tempStr = sub(tempStr, 1, pos1 - 1) .. sub(tempStr, pos2 + 1)
	until false

	tempStr = chatStr
	
	repeat
		pos1, pos2 = find(tempStr, "|Hachievement:", 1, true)
		
		if not pos1 then break end

		achieveID[#achieveID + 1] = sub(tempStr, pos1 + 14, find(tempStr, ":", pos1 + 15, true) - 1, true)
		
		tempStr = sub(tempStr, 1, pos1 - 1) .. sub(tempStr, pos2 + 1)
	until false
	
	chatStr = lower(chatStr)

	for word in gmatch(chatStr, "%S+") do
		chatData[#chatData + 1] = word
	end
	
	-- now cycle through filter, tokenize, and then simplify
	
	local filterPos	= 0
	local filterLen	= string.len(filterStr)
	local token	= ""
	local tokenData	= ""
	local resTable  = {}
	local c
	local result = ""
	local tempPos	
	local found
	local gotPR, gotPL = false
	
	while filterPos < filterLen do
		
		if gotPL == true then
			c     = " "
			token = "("
			gotPL = false
		elseif gotPR == true then
			c     = " "
			token = ")"
			gotPR = false
		else	
			filterPos = filterPos + 1
			
			c = strbyte(filterStr, filterPos)
	
			if c < 32 or c > 126 then
				c = char(c)
			else
				c = lower(char(c))			
			end
		end
	
		if c == " " or c == "(" or c == ")" or filterPos == filterLen then
			if c == "(" then
				gotPL = true
			elseif c == ")" then
				gotPR = true
			elseif filterPos == filterLen and c ~= " " then
				token = token .. c
			end
			
			if token ~= "" then
				tempPos = find(token, "=", 1, true)
			
				if tempPos then
					tokenData = sub(token, tempPos+1, strlen(token)-1)
					token	  = sub(token, 1, tempPos-1).."]"
				else
					tokenData = ""
				end
			
				if token == "(" then
					result = result..token
				elseif token == ")" then
					result = result..token
				elseif token == "not" then
					result = result .. "!"
				elseif token == "and" then
				elseif token == "or" then
					result = result.."|"
				elseif token == "[word]" then
					found = false

					for count = 1, #chatData do
						if chatData[count] == tokenData then
							found = true
							break
						end
					end
				
					if found == true then
						result = result .. "T"
					else
						result = result .. "F"					
					end
				elseif token == "[contains]" then
					if string.find(chatStr, tokenData, 1, true) then
						result = result .. "T"						
					else
						result = result .. "F"									
					end				
				elseif token == "[item]" then
					if tokenData == "" then
						if #itemID > 0 then
							result = result .. "T"						
						else
							result = result .. "F"
						end
					else
						found = false
					
						for count = 1, #itemID do
							if tokenData == itemID[count] then
								found = true
								break
							end
						end
					
						if found == true then
							result = result .. "T"						
						else
							result = result .. "F"						
						end
					end
				elseif token == "[spell]" then
					if tokenData == "" then
						if #spellID > 0 then
							result = result .. "T"
						else
							result = result .. "F"						
						end
					else
						found = false
						
						for count = 1, #spellID do
							if tokenData == spellID[count] then
								found = true
								break
							end
						end
						
						if found == true then
							result = result .. "T"						
						else
							result = result .. "F"
						end
					end
				elseif token == "[achievement]" then
					if tokenData == "" then
						if #achieveID > 0 then
							result = result .. "T"
						else
							result = result .. "F"
						end
					else
						found = false
					
						for count = 1, #achieveID do
							if tokenData == achieveID[count] then
								found = true
								break
							end
						end
					
						if found == true then
							result = result .. "T"						
						else
							result = result .. "F"						
						end
					end
				elseif token == "[link]" then
					if #achieveID > 0 or #spellID > 0 or #itemID > 0 then
						result = result .. "T"
					else
						result = result .. "F"
					end				
				else
					return -1
				end
			
				token = ""
			end
		else
			token = token .. c
		end
	end
		
	local P1,P2,P3 = 1
		
	repeat
		found = false	
	
		for count = 1, string.len(result) do
			c = strbyte(result, count)
			
			if c == 40 then
				P1 = count
			elseif c == 41 then
				P2      = count
				found   = true
				tempStr = sub(result, P1 + 1, P2 - 1)
				
				P3 = string.find(tempStr, "!", 1, true)
				
				while P3 do
					if strbyte(tempStr, P3 + 1) == 70 then
						c = 84
					else
						c = 70
					end
					
					tempStr = sub(tempStr, 1, P3 - 1) .. char(c) .. sub(tempStr, P3+2)
					P3 = find(tempStr, "!", 1, true)
				end
								
				P3 = string.find(tempStr, "|", 1, true)
				
				while P3 do
					if strbyte(tempStr, P3 - 1) == 84 or strbyte(tempStr, P3 + 1) == 84 then
						c = 84
					else
						c = 70
					end
						
					tempStr = sub(tempStr, 1, P3 - 2) .. char(c) .. sub(tempStr, P3+2)
					P3 = find(tempStr, "|", 1, true)
				end
								
				result = sub(result, 1, P1 - 1) .. tempStr .. sub(result, P2 + 1)
				
				break
			end
		end
		
	until found == false
		
	if find(result, "F", 1, true) then
		return false
	else
		return true
	end	
end

----------------------------
-- CHAT FILTERING HANDLER --
----------------------------

local function chatMessageFilter (self, event, message, from, ...)

	--print ("chatMsg evt=" .. (event or "nil") .. " msg=".. (message or "nil") .. " from=" .. (from or "nil"))
	
	if addonLoaded ~= true then
		return false
	end
	

	if event == "CHAT_MSG_MONSTER_EMOTE" or event == "CHAT_MSG_MONSTER_PARTY" or event == "CHAT_MSG_MONSTER_SAY" or
	   event == "CHAT_MSG_MONSTER_WHISPER" or event == "CHAT_MSG_MONSTER_YELL" then

        	return (hasNPCIgnored(from) > 0)
		
	elseif event == "CHAT_MSG_SYSTEM" then
		
		if message == ERR_IGNORE_NOT_FOUND and GetTime() - loadedTime < 60 then
			return true
		end

		if message == ERR_IGNORE_ALREADY_S and GetTime() - loadedTime < 60 then
			return true
		end
			
		if message == ERR_IGNORE_FULL then
			return true
		end
			
		local pName = ""

		for count = 1, #GlobalIgnoreDB.ignoreList do
			
			if GlobalIgnoreDB.typeList[count] == "server" then
			
				if string.find(message, "-"..GlobalIgnoreDB.ignoreList[count], 1, true) ~= nil then
					return true
				end
			else
				if serverName == getServer(GlobalIgnoreDB.ignoreList[count]) then
					pName = removeServer(GlobalIgnoreDB.ignoreList[count])
				else
					pName = GlobalIgnoreDB.ignoreList[count]
				end
				

				local msgOffline = strDown(string.format(ERR_FRIEND_OFFLINE_S, pName))
				local msgOnline  = strDown(string.format(ERR_FRIEND_ONLINE_SS, pName, pName))

				message = strDown(message)
					
				if (message == msgOffline) or (message == msgOnline) then
					return true
				end
	  		end
	  	end
			
		return false

	elseif (from ~= nil) and (from ~= "") then
		
		local idx = string.find(from, "-", 1, true)
		
		if idx == nil then
			from = from .. "-" .. serverName
		else
			from = string.sub(from, 1, idx - 1) .. "-" .. Proper(string.sub(from, idx + 1, string.len(from)))		
		end
				
		if hasGlobalIgnored(from) > 0 or hasServerIgnored(getServer(from)) > 0 then			
			
			if event == "CHAT_MSG_WHISPER" then
				local temp = from .. math.ceil(GetTime() - 0.5)
				
				if (temp ~= lastSentIgnore) then
					SendChatMessage(L["MSG_1"], "WHISPER", nil, from)
					lastSentIgnore = temp
				end
			end

			return true
		else
	
			if GlobalIgnoreDB.spamFilter == true then
						
				for count = 1, #GlobalIgnoreDB.filterList do
				
					if filterComplex(GlobalIgnoreDB.filterList[count], message) == true then
						if GlobalIgnoreDB.invertSpam == true then
							return false
						else
							GlobalIgnoreDB.filterTotal        = GlobalIgnoreDB.filterTotal + 1
							GlobalIgnoreDB.filterCount[count] = GlobalIgnoreDB.filterCount[count] + 1
							
							return true
						end
					end
				end
				
				if GlobalIgnoreDB.invertSpam == true then
					GlobalIgnoreDB.filterTotal = GlobalIgnoreDB.filterTotal + 1
					return true
				end
				
			end
		end
	end		  	
	
	return false
end

local chatEvents = (
		{
		"CHAT_MSG_ACHIEVEMENT",
		"CHAT_MSG_BATTLEGROUND",
		"CHAT_MSG_BATTLEGROUND_LEADER",
		"CHAT_MSG_CHANNEL",
		"CHAT_MSG_CHANNEL_JOIN",
		"CHAT_MSG_CHANNEL_LEAVE",
		"CHAT_MSG_EMOTE",
		"CHAT_MSG_GUILD",
		"CHAT_MSG_GUILD_ACHIEVEMENT",	
		"CHAT_MSG_MONSTER_EMOTE",
		"CHAT_MSG_MONSTER_PARTY",
		"CHAT_MSG_MONSTER_SAY",
		"CHAT_MSG_MONSTER_WHISPER",
		"CHAT_MSG_MONSTER_YELL",
		"CHAT_MSG_OFFICER",
		"CHAT_MSG_PARTY",
		"CHAT_MSG_RAID",
		"CHAT_MSG_RAID_LEADER",
		"CHAT_MSG_RAID_WARNING",
		"CHAT_MSG_SAY",
		"CHAT_MSG_SYSTEM",
		"CHAT_MSG_TEXT_EMOTE",
		"CHAT_MSG_WHISPER",
		"CHAT_MSG_WHISPER_INFORM",
		"CHAT_MSG_YELL"
		}
	)

for key, value in pairs (chatEvents) do
	ChatFrame_AddMessageEventFilter(value, chatMessageFilter)
end

-------------------
-- CHAT COMMANDS --
-------------------

function SlashCmdList.GIGNORE (msg)

	msg = strDown(msg)
	
	local args   = {}
	local argStr = ""
	local count  = 1

	local str = GetWord(msg, count)
	
	while str ~= "" do
		
		table.insert(args, str)
		
		if count == 2 then
			argStr = str
		elseif count > 2 then
			argStr = argStr .. " ".. str
		end
		
		count = count + 1
		str   = GetWord(msg, count)
	end
	
	if args[1] == "test" then

	elseif args[1] == "clear" then
	
		if firstClear and args[2] ~= nil and args[2] == "confirm" then	
			ResetIgnoreDB()
			ShowMsg(L["CMD_2"])
			SyncIgnoreList(GlobalIgnoreDB.chatmsg == nil or GlobalIgnoreDB.chatmsg == false)
			firstClear = false
		else
			ShowMsg("|cffff0000" .. L["CMD_1"])
			firstClear = true
		end
		
	elseif args[1] == "defexpire" then
	
		if tonumber(args[2]) then
			GlobalIgnoreDB.defexpire = tonumber(args[2])
			
			ShowMsg (format(L["CMD_3"], GlobalIgnoreDB.defexpire, dayString(GlobalIgnoreDB.defexpire)))
		end
		
	elseif msg == "asknote true" or msg == "asknote on" then
	
		GlobalIgnoreDB.asknote = true
		ShowMsg (L["CMD_4"])

	elseif msg == "asknote false" or msg == "asknote off" then

		GlobalIgnoreDB.asknote = false
		ShowMsg (L["CMD_5"])
		
	elseif msg == "showmsg true" or msg == "showmsg on" then
	
		GlobalIgnoreDB.chatmsg = true
		ShowMsg (L["CMD_6"])

	elseif msg == "showmsg false" or msg == "showmsg off" then
	
		GlobalIgnoreDB.chatmsg = false	
		ShowMsg (L["CMD_7"])
	
	elseif msg == "sameserver true" or msg == "sameserver on" then
	
		GlobalIgnoreDB.sameserver = true
		ShowMsg(L["CMD_10"])

	elseif msg == "sameserver false" or msg == "sameserver off" then
	
		GlobalIgnoreDB.sameserver = false
		ShowMsg(L["CMD_11"])
		
	elseif args[1] == "list" then
	
		ShowIgnoreList(argStr)
		
	elseif (args[1] == "add" or args[1] == "ignore") then
	
		argStr = (trim(Proper(argStr)) or "")
		
		if argStr == "" then
			argStr, server =  UnitName("target")
					
			if server ~= nil then
				argStr = argStr .. "-"..server
			end
			
			if argStr == nil or not UnitPlayerControlled("target") then
				argStr = ""
			end
		end
		
		if argStr ~= "" then
			AddIgnore (Proper(argStr))
		end
		
	elseif (args[1] == "remove" or args[1] == "delete") and args[2] ~= nil and args[2] ~= "" then
	
		if tonumber(argStr) then
		
			local str = GlobalIgnoreDB.typeList[tonumber(argStr)]
			
			if str == "npc" then
				ShowMsg (format(L["CMD_12"], Proper(GlobalIgnoreDB.ignoreList[tonumber(argStr)], true)))
				RemoveFromList(tonumber(argStr))
				GILUpdateUI(true)
			else
				DelIgnore(args[2], true)
			end
		else
			argStr = Proper(argStr, true)
			
			local npcIndex = hasNPCIgnored(argStr)
		
			if npcIndex > 0 then
				ShowMsg (format(L["CMD_12"], argStr))
				RemoveFromList(npcIndex)
				GILUpdateUI(true)
			else	
				DelIgnore (args[2], true)
			end
		end
		
	elseif (args[1] == "server" or args[1] == "addserver") and args[2] ~= nil and args[2] ~= "" then

		AddOrDelServer(argStr)
		GILUpdateUI(true)

	elseif (args[1] == "npc" or args[1] == "addnpc") then
	
		AddOrDelNPC(argStr)
		GILUpdateUI(true)
		
	elseif args[1] == "expire" and args[2] ~= nil and args[2] ~= "" and tonumber(args[3]) then

		if tonumber(args[2]) then
			local index = tonumber(args[2])

			if (index > 0) and (index <= #GlobalIgnoreDB.ignoreList) then
			
				GlobalIgnoreDB.expList[index] = tonumber(args[3])
				ShowMsg(format(L["CMD_14"], GlobalIgnoreDB.ignoreList[index], tonumber(args[3])))
			end

		else
			local name        = Proper(addServer(args[2]))
			local playerIndex = hasGlobalIgnored(name)

			if playerIndex > 0 then
				GlobalIgnoreDB.expList[playerIndex] = tonumber(args[3])
				ShowMsg(format(L["CMD_14"], name, tonumber(args[3])))
			end
		end
		
	elseif args[1] == "gui" then	
	
		GUI()
		
	elseif args[1] == "sync" then
	
		SyncIgnoreList()
		
	elseif args[1] == "dellist" then
	
		for count = 1, #GlobalIgnoreDB.delList do
			print(GlobalIgnoreDB.delList[count])
		end
		
	elseif args[1] == "prune" then
	
		if args[2] == "confirm" and firstPrune == true then
		
			PruneIgnoreList(pruneDays, true)
			
			firstPrune = false
		elseif args[2] == nil or tonumber(args[2]) == nil then
		
			ShowMsg(L["CMD_15"])
		else
			if firstPrune == false then

				pruneDays = tonumber(args[2])
				
				ShowMsg(format(L["CMD_16"], pruneDays))
				ShowMsg(format(L["CMD_17"], PruneIgnoreList(pruneDays, false)))
				
				firstPrune = true
			end
		end

	else
		ShowMsg (L["HELP_1"])
		ShowMsg ("")
		ShowMsg ("  " .. L["HELP_2"])
		ShowMsg ("  " .. L["HELP_3"])
		ShowMsg ("  " .. L["HELP_4"])
		ShowMsg ("  " .. L["HELP_5"])
		ShowMsg ("  " .. L["HELP_6"])
		ShowMsg ("  " .. L["HELP_7"])
		ShowMsg ("  " .. L["HELP_8"])
		ShowMsg ("  " .. L["HELP_15"])
		ShowMsg ("  " .. L["HELP_16"])		
		ShowMsg ("")
		ShowMsg ("  " .. format(L["HELP_10"], OnOff(GlobalIgnoreDB.chatmsg)))
		ShowMsg ("  " .. format(L["HELP_11"], OnOff(GlobalIgnoreDB.sameserver)))
		ShowMsg ("  " .. format(L["HELP_12"], GlobalIgnoreDB.defexpire))
		ShowMsg ("  " .. format(L["HELP_13"], OnOff(GlobalIgnoreDB.asknote)))
		ShowMsg ("")
		ShowMsg (L["HELP_14"])
	end
end

---------------------------
-- IGNORE FUNCTION HOOKS --
---------------------------

BlizzardAddIgnore      = AddIgnore
BlizzardDelIgnore      = DelIgnore
BlizzardAddOrDelIgnore = AddOrDelIgnore

AddIgnore = function(name, noNote)

	if (not name or name == "") then

		name, server = UnitName("target")
					
		if server ~= nil then
			name = name .. "-"..server
		end
	end
	
	if (not name or name == "") then
		return
	end

	local note  = ""
	local space = string.find(name, " ")
	local days  = GlobalIgnoreDB.defexpire
	
	if space then
		note = string.sub(name, space + 1)
		name = string.sub(name, 0, space - 1)
		
		space = string.find(note, " ")
		
		if space then
			if tonumber(string.sub(note, 0, space - 1)) then
				days = tonumber(string.sub(note, 0, space - 1))
				note = string.sub(note, space + 1)
			end
		else
			if tonumber(note) then
				days = tonumber(note)
				note = ""
			end
		end
	end
	
	needSorted = true
	name	   = Proper(addServer(name))
		
	if Proper(addServer(UnitName("player"))) ~= name then
	
		local index = hasGlobalIgnored(name)
	
		if index == 0 then
			AddToList(name, faction, note)
			
			if GlobalIgnoreDB.asknote == true and not noNote then
			
				nameUI = name				
				StaticPopup_Show("GIL_REASON", nameUI)			
			end
			
			ShowMsg(format(L["ADD_2"], name))
			
			if GetNumIgnores() < maxIgnoreSize then
				BlizzardAddIgnore(removeServer(name))
			end
		else
			if hasIgnored(name) > 0 then
				ShowMsg(format(L["ADD_1"], name))
			end
		
			BlizzardAddIgnore(removeServer(name))
		end
		
		removeDeleted(name)
		
		indentUI = 0
		
		GILUpdateUI()
	else
		ShowMsg(L["ADD_3"])
	end	
end

DelIgnore = function(idxpos, isGIL)

	local name = ""
	
	if isGIL then
		if tonumber(idxpos) ~= nil then
			name = GlobalIgnoreDB.ignoreList[tonumber(idxpos)]
			
			if name == nil then
				return
			end
		else
			name = idxpos
		end
	else
		if tonumber(idxpos) ~= nil then			
			name = GetIgnoreName(idxpos)
		else
			name   = idxpos
			idxpos = hasIgnored(name)
		end
	end
	
	needSorted = true	
	name 	   = Proper(addServer(name))

	addDeleted(name)
	
	local index = hasGlobalIgnored(name)

	if index > 0 then
		ShowMsg(format(L["REM_1"], name))
		
		RemoveFromList(index)		
		
		name = removeServer(name)
		
		if hasIgnored(name) > 0 then
			BlizzardDelIgnore(name)
		else
			GILUpdateUI()
		end
	else
		BlizzardDelIgnore(idxpos)
	end
		
	indentUI = 0
end

AddOrDelIgnore = function(name)

	if (not name or name == "") then

		name, server = UnitName("target")
					
		if server ~= nil then
			name = name .. "-"..server
		end
	end
	
	if (not name or name == "") then
		return
	end
	
	name = Proper(addServer(name))

	local index = hasGlobalIgnored(name)

	if index == 0 then
		AddIgnore(name)
	else
		DelIgnore(index, true)
	end	
end

AddOrDelNPC = function (argStr)

	if tonumber(argStr) then
	
		local nIndex = tonumber(argStr)
		
		if (nIndex > 0) and (nIndex <= #GlobalIgnoreDB.ignoreList) and (GlobalIgnoreDB.typeList[nIndex] == "npc") then
				
			ShowMsg (format(L["CMD_12"], GlobalIgnoreDB.ignoreList[nIndex]))
			RemoveFromList(nIndex)
		end
	else
				
		argStr = (trim(Proper(argStr, true)) or "")
	
		if argStr == "" then
			argStr = Proper(UnitName("target"), true)
				
			if argStr == nil or UnitPlayerControlled("target") then
				argStr = ""
			end
		end
			
		if argStr ~= "" then

			local npcIndex = hasNPCIgnored(argStr)
		
			if npcIndex > 0 then
				local name = GlobalIgnoreDB.ignoreList[npcIndex]
			
				ShowMsg (format(L["CMD_12"], name))
				RemoveFromList(npcIndex)
			else
				ShowMsg (format(L["CMD_13"], argStr))
				AddToList(argStr, faction, "", "npc")
			end
		end
	end
end

AddOrDelServer = function (sName)

	if not sName then return end

	if tonumber(sName) then
	
		local sIndex = tonumber(sName)
		
		if (sIndex > 0) and (sIndex <= #GlobalIgnoreDB.ignoreList) and (GlobalIgnoreDB.typeList[sIndex] == "server") then
		
			ShowMsg(format(L["CMD_19"], GlobalIgnoreDB.ignoreList[sIndex]))
			RemoveFromList(sIndex)
		end
	
	else
	
		sName = Proper(sName)
		
		local sIndex = hasServerIgnored(sName)
		
		if sIndex > 0 then
		
			ShowMsg(format(L["CMD_19"], sName))
			RemoveFromList(sIndex)
	
		else
		
			ShowMsg(format(L["CMD_18"], sName))
			AddToList(sName, faction, "", "server")
		end
	end
end

-----------------------------
-- Global Ignore List Main --
-----------------------------

local frame = CreateFrame("FRAME")

frame:RegisterEvent("IGNORELIST_UPDATE")
frame:RegisterEvent("PLAYER_LOGIN")
frame:RegisterEvent("ADDON_LOADED")

SLASH_GIGNORE1 = "/gignore"
SLASH_GIGNORE2 = "/gi"
SLASH_GIGNORE3 = "/gil"

frame:SetScript("OnEvent", EventHandler)

-------------
-- THE END --
-------------